IMPLEMENTATION MODULE StandardMenu;

<* M2EXTENSIONS + *> <* STORAGE + *>

(*
	Created with TAB size 3
	Go to Configure > Editor to adjust TAB size if needed
*)

IMPORT Wimdows;
FROM Lib IMPORT Delay;

(* -------------- types (public) ----------------

TYPE
	
     ButtonType = RECORD
	  					  withMouseOver : BOOLEAN;   (* button changes look when mouse is over it *)
	                 withMask : BOOLEAN;        (* button is drawn with a bitmap mask *)
						END;

     BitmapName = ARRAY[1..30] OF CHAR;
     ButtonStrings = RECORD
	                    normal, mouseOver, normalMask, mouseOverMask : BitmapName;
							END;
							
     ButtonRc = RECORD
                  x0,y0,width,height : INTEGER;  			   (* coordinates, width and height of button *)
						visible : BOOLEAN;                        (* if a button is not visible, it will not be drawn and cannot be clicked *)
						type : ButtonType;                        (* should be drawn with mask or not *)
						bitmap, mask: ARRAY BOOLEAN OF CARDINAL;	(* bitmap & mask, if the mouse is hovering over the button or not *)	
					 END;
							
	  						
*)
(*--------- left click memory (private) --------- *)
TYPE
	ClickPt = POINTER TO ClickRc;
	ClickRc = RECORD
					x,y : INTEGER;
				 END;

VAR
	leftClick : ClickPt;
	
PROCEDURE RegisterLeftClick(x,y:INTEGER);
BEGIN
   (* we only remember the last click *)
	IF (leftClick = NIL) THEN
		NEW(leftClick);
	END;
	leftClick^ := ClickRc{x,y};
END RegisterLeftClick;

(* ----------- menu procedures (public) ---------- *)
PROCEDURE CreateButtonX(w, h : INTEGER; flags : ButtonType; bitmaps : ButtonStrings): ButtonRc;
VAR button : ButtonRc;
BEGIN
	button.bitmap[FALSE] := Wimdows.LoadBitmap(bitmaps.normal,w,h);
	IF (flags.withMask) THEN
		button.mask[FALSE] := Wimdows.LoadBitmap(bitmaps.normalMask,w,h);
	END;
	IF (flags.withMouseOver) THEN
		button.bitmap[TRUE] := Wimdows.LoadBitmap(bitmaps.mouseOver,w,h);
		IF (flags.withMask) THEN
			IF (bitmaps.mouseOverMask[1] = '=') THEN
				button.mask[TRUE] := button.mask[FALSE];
			ELSE
				button.mask[TRUE] := Wimdows.LoadBitmap(bitmaps.mouseOverMask,w,h);
			END;
		END;
	END;
   button.width := w;
	button.height := h;	
	button.type := flags;
	button.visible := TRUE;
	RETURN button;
END CreateButtonX;

(* simple procedures *)
PROCEDURE CreateButton(w, h : INTEGER; normal, mouseOver: BitmapName): ButtonRc;
BEGIN
	RETURN CreateButtonX(w, h, ButtonType{TRUE,FALSE}, ButtonStrings{normal, mouseOver, " ", " "});
END CreateButton;

PROCEDURE CreateMaskedButton(w, h : INTEGER; normal, mouseOver, mask: BitmapName): ButtonRc;
BEGIN
	RETURN CreateButtonX(w, h, ButtonType{TRUE,TRUE}, ButtonStrings{normal, mouseOver, mask, "="});
END CreateMaskedButton;

PROCEDURE CreateField(w, h : INTEGER; normal, mask: BitmapName): ButtonRc;
BEGIN
	RETURN CreateButtonX(w, h, ButtonType{FALSE,TRUE}, ButtonStrings{normal, " ", mask, " "});
END CreateField;


PROCEDURE SetVerticalMenu(x0, y0, yGap:INTEGER; VAR buttons : ARRAY OF ButtonRc);
VAR i : CARDINAL;
BEGIN
	buttons[0].x0 := x0;
	buttons[0].y0 := y0;
	FOR i := 1 TO HIGH(buttons) DO
		buttons[i].x0 := buttons[i-1].x0;
		buttons[i].y0 := buttons[i-1].y0 + yGap;
	END;
END SetVerticalMenu;	

PROCEDURE InButton(button : ButtonRc; x,y:INTEGER):BOOLEAN;
BEGIN
   RETURN (x > button.x0) AND (x < button.x0 + button.width) AND (y > button.y0) AND (y < button.y0 + button.height);
END InButton;

PROCEDURE DrawButton(button : ButtonRc);
VAR x,y : INTEGER;
    mouseover : BOOLEAN;
BEGIN
	IF (button.visible) THEN
		IF (button.type.withMouseOver) THEN
			Wimdows.MousePos(x,y);
  		   mouseover := InButton(button,x,y);
	  	ELSE
		   mouseover := FALSE;
	  	END;
		IF (button.type.withMask) THEN	
			Wimdows.DrawMaskBitmap(button.bitmap[mouseover],button.mask[mouseover],button.x0,button.y0);
	  	ELSE
			Wimdows.DrawBitmap(button.bitmap[mouseover],button.x0,button.y0);
	  	END;
	END;
END DrawButton;

PROCEDURE ButtonSelected(button : ButtonRc):BOOLEAN;
 BEGIN
   IF (button.visible) THEN
 	  Wimdows.OnMouseLeftUp(RegisterLeftClick); 	
	  IF ((leftClick <> NIL) AND InButton(button,leftClick^.x,leftClick^.y)) THEN
		  DISPOSE(leftClick);
		  RETURN TRUE;
	  END;
	END;
	RETURN FALSE;
END ButtonSelected;

PROCEDURE MenuSelection(menu : ARRAY OF ButtonRc):INTEGER;
 VAR i : CARDINAL;
 BEGIN
	LOOP
	   Wimdows.UpdateWindow();
		FOR i := 0 TO HIGH(menu) DO
		  	DrawButton(menu[i]);
			IF (ButtonSelected(menu[i])) THEN
				RETURN i;
			END;		
 	  END;
	  Delay(25);
	END;
END MenuSelection;


END StandardMenu.