MODULE StandardMenuExamples;

IMPORT Wimdows;
FROM IO IMPORT WrStr,WrCard,WrLn;
FROM Lib IMPORT Delay;
FROM StandardMenu IMPORT ButtonRc, CreateButton, CreateMaskedButton, CreateField, SetVerticalMenu, DrawButton, MenuSelection;

(* This module contains two examples of how StandardMenu can be used:
   - The main menu is a simple example with 3 buttons "start", "settings" and "quit"
	- The settings menu is a more complex example with more user interaction
*)

(* ----- init procedures ------ *)

CONST SCREENX = 500;
      SCREENY = 500;

PROCEDURE LoadStartMenu(VAR startMenu : ARRAY OF ButtonRc);
(* in this menu, all buttons have same heigth and width *)
CONST
	w = 250;
	h = 75;
BEGIN
	
	startMenu[0] := CreateButton(w, h, "images/start0.bmp", "images/start1.bmp");
	startMenu[1] := CreateButton(w, h, "images/settings0.bmp","images/settings1.bmp");
	startMenu[2] := CreateButton(w, h, "images/quit0.bmp", "images/quit1.bmp");
	
	SetVerticalMenu(110, 50, 150, startMenu);

END LoadStartMenu;

CONST SPINNER_DOWN = 0;  (* had hier ook een enumeratie kunnen gebruiken *)
      SPINNER_UP = 1;
      SPINNER_BACK = 2;  		
		MAX_DIFFICULTY = 2;

PROCEDURE LoadDifficultySelector(VAR spinnerMenu : ARRAY OF ButtonRc; VAR spinner: ARRAY OF ButtonRc);
VAR i : CARDINAL;
CONST
	w = 200;  (* width of spinner *)
	wb = 50;  (* width of up/down buttons *)
	h =  100;
	
	X0 = 25;
	Y0 = 250;
	
	XBACK = 325;
	YBACK = 350;
BEGIN

  spinnerMenu[SPINNER_DOWN] := CreateMaskedButton(wb, h, "images/down.bmp", "images/down1.bmp", "images/down_mask.bmp");
  spinnerMenu[SPINNER_DOWN].x0 := X0 ;
  spinnerMenu[SPINNER_DOWN].y0 := Y0 ;

  spinnerMenu[SPINNER_UP] :=  CreateMaskedButton(wb, h, "images/up.bmp", "images/up1.bmp", "images/up_mask.bmp");
  spinnerMenu[SPINNER_UP].x0 := X0 + wb + w;
  spinnerMenu[SPINNER_UP].y0 := Y0;

  spinnerMenu[SPINNER_BACK] := CreateMaskedButton(200, 100, "images/back0.bmp", "images/back1.bmp", "images/back_mask.bmp");
  spinnerMenu[SPINNER_BACK].x0 := XBACK;
  spinnerMenu[SPINNER_BACK].y0 := YBACK;

  spinner[0] := CreateField(w, h, "images/easy.bmp", "images/easy_mask.bmp");
  spinner[1] := CreateField(w, h, "images/medium.bmp", "images/medium_mask.bmp");
  spinner[2] := CreateField(w, h, "images/hard.bmp", "images/hard_mask.bmp");

  FOR i:= 0 TO MAX_DIFFICULTY DO
    spinner[i].x0 := X0 + wb;
    spinner[i].y0 := Y0;
  END;

END LoadDifficultySelector;

(* ----- game procedures ------ *)


PROCEDURE StartGame();
BEGIN
	Wimdows.ClearScreen();
	Wimdows.SetFont(0,Wimdows._clrBRIGHTWHITE);
	Wimdows.Text("you're now playing the game!", 175, 250);
	Wimdows.UpdateWindow();
	Delay(1500);
END StartGame;

VAR spinnerMenu : ARRAY [0..2] OF ButtonRc;
    difficulties : ARRAY [0..2] OF ButtonRc;
	 background : INTEGER;

PROCEDURE Select(VAR level : CARDINAL);
VAR select : INTEGER;
    diffbg : CARDINAL;
	 w, h, diffbgx, diffbgy : CARDINAL;
BEGIN
	(* we maken een tijdelijke bitmap als achtergrond van difficulty level bitmap
		zodat we niet altijd hele scherm moeten hertekenen als difficulty veranderd *)
	w := difficulties[0].width;
	h := difficulties[0].height;
	diffbgx := difficulties[0].x0;
	diffbgy := difficulties[0].y0;
	diffbg := Wimdows.CreateBitmap(w, h);
	Wimdows.CopyBitmap(background, diffbg, diffbgx, diffbgy, 0, 0, w, h);
	
	(* "hoofdlus": wacht tot een knop is ingedrukt en verander difficuly level indien nodig *)
	Wimdows.DrawBitmap(background, 0, 0);	
	LOOP
		Wimdows.DrawBitmap(diffbg, diffbgx, diffbgy);
		DrawButton(difficulties[level]);
		Wimdows.UpdateWindow();
		select := MenuSelection(spinnerMenu);
		CASE select OF
			SPINNER_DOWN  : IF (level > 0) THEN
									level := level - 1;
								END;
			|SPINNER_UP   : IF (level < MAX_DIFFICULTY) THEN
									level := level + 1;
								END;
			|SPINNER_BACK : EXIT; (* exit LOOP *)
			ELSE
				WrStr("error! button undefined!"); WrLn;
		END; 		
	END;
	
	(* geheugen vrijmaken *)
	Wimdows.DestroyBitmap(diffbg);

END Select;

PROCEDURE Quit();
BEGIN
  WrStr("quit"); WrLn;
END Quit;

(* ----- main program ------ *)


VAR select : INTEGER;
    startMenu: ARRAY [0..2] OF ButtonRc;
	 difficultyLevel : CARDINAL;

BEGIN

  Wimdows.Window("Dragon Fighter",SCREENX, SCREENY);
  Wimdows.SetBuffering(TRUE);

  LoadStartMenu(startMenu);
  LoadDifficultySelector(spinnerMenu,difficulties);
  difficultyLevel := 0;

  background := Wimdows.LoadBitmap("images/dragon.bmp",SCREENX, SCREENY);

  (* hoofdlus: wacht tot een knop wordt ingedrukt en doe dan iets *)
  LOOP (* main menu *)
    Wimdows.ClearScreen();
	 Wimdows.DrawBitmap(background, 0, 0);
	 Wimdows.UpdateWindow();
  	 select := MenuSelection(startMenu);
    CASE select OF
        0 : StartGame();  |
		  1 : Select(difficultyLevel);     |
	     2 : Quit(); EXIT;   (* exit LOOP (niet het programma) *)
	 ELSE
			WrStr("error! wrong button number!"); WrLn;
    END;
  END;

  Wimdows.KillWindow;

END StandardMenuExamples.